// -*-c++-*-
//===========================================================
//=     University of Illinois at Urbana/Champaign          =
//=     Department of Computer Science                      =
//=     Dr. Dan Roth - Cognitive Computation Group          =
//=                                                         =
//=  Project: Snow                                          =
//=                                                         =
//=   Module: LearningAlgorithm.h                           =
//=  Version: 2.1                                           =
//=  Authors: Jeff Rosen and Andrew Carlson                 =
//=     Date: xx/xx/99                                      = 
//=                                                         =
//= Comments:                                               =
//===========================================================

#ifndef __LEARNINGALGORITHM_H__
#define __LEARNINGALGORITHM_H__

#include "Example.h"
#include "TargetIdSet.h"

class Target;

class LearningAlgorithm
{
public:
    LearningAlgorithm();
    LearningAlgorithm( double threshold, double defaultWeitght );
    
    virtual bool PresentExample( Target& tar, Example& ex );
    virtual void TrainingComplete( Target& tar );
    virtual void Discard( Target& tar );

    virtual void PrepareToRank();
    virtual void SetTargetActivation( Target& tar, Example& ex );
    virtual double ReturnNormalizedActivation( Target& tar);
    
    virtual void Show( ostream& out ) = 0;
    
    virtual void Read( ifstream& in ) = 0;
    virtual void Write( ofstream& out ) = 0;

    int     index;
    TargetIdSet targetIds;
    double  threshold;
    double  defaultWeight;
};

inline LearningAlgorithm::LearningAlgorithm() :
        threshold(1.0),
        defaultWeight(0.1)
{
}

inline LearningAlgorithm::LearningAlgorithm( double th, double dw ) :
        threshold(th),
        defaultWeight(dw)
{
}

inline bool LearningAlgorithm::PresentExample( Target& tar, Example& ex )
{
    cout << "LearningAlgorithm::PresentExample()..." << endl;
    return false;
}

inline void LearningAlgorithm::TrainingComplete( Target& tar )
{
    cout << "LearningAlgorithm::TrainingComplete()..." << endl;
}

inline void LearningAlgorithm::Discard( Target& tar )
{
    cout << "LearningAlgorithm::Discard()..." << endl;
}

inline void LearningAlgorithm::PrepareToRank()
{
}

inline void LearningAlgorithm::SetTargetActivation( Target& tar, Example& ex )
{
    cout << "LearningAlgorithm::SetTargetActivation()..." << endl;
}

inline double LearningAlgorithm::ReturnNormalizedActivation( Target& tar )
{
    cout << "LearningAlgorithm::ReturnNormalizedActivation()..." << endl;
    return 0;
}

#endif

